//*****************************************************************************
// File: command.cpp
// Description: Nirva command classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#include "nvcc.h"


using namespace std;


/////////////////////////////////////////////////////////////////////////////
// NvServerCommand

NvServerCommand::NvServerCommand()
{
	Clear();
}

NvServerCommand::~NvServerCommand()
{
	Clear();
}



void NvServerCommand::Clear()
{
	m_Error.SetError();
	m_sService = "";
	m_sClass = "";
	m_sCommand = "";
	m_sInContainer = "";
	m_sOutContainer = "";
	m_sLanguage = "";

	m_Parameters.RemoveAll();
}



void NvServerCommand::ParseParameters(const char *Parameters, NvObjIndStringList* Variables, char sep)
{
	m_Parameters.RemoveAll();

	if(Parameters == NULL)
		return;
	
	string Param = Parameters;
	
//	CleanString(Param);
	string::size_type index;
	string Parameter = "";
	string Value = "";
	string PartialValue = "";
	bool stopit = false;

	bool bAutoSep = false;
	if(sep==0)
		bAutoSep = true;

	while(1)
	{
		Parameter = "";
		Value = "";
		index = Param.find('=');
		if(index == string::npos)
			break;
		Parameter = Param.substr(0, index);
		Param = Param.substr(index+1);

		RemoveFirstAndEndBlanks(Parameter);
		RemoveFirstAndEndBlanks(Param);
		if(bAutoSep && Param.length())
			sep = Param[0];

		index = Param.find(sep);
		if(index == string::npos)
			break;
		Param = Param.substr(index+1);
		
		stopit = false;
		while(1)
		{
			index = 0;
			// We search for the next occurence of a " character that is not doubled
			// This marks the end of the value
			while(1)
			{
				if(index >= Param.size())
				{
					index = string::npos;
					stopit = true;
					break;
				}
				index = Param.find(sep, index);
				if(index == string::npos)
				{
					stopit = true;
					break;
				}
				if((index+1) >= Param.size())
					break;
				if(Param[index+1] != sep)
					break;
				index++;
				index++;
			}
			
			if(stopit)
				break;
			PartialValue = Param.substr(0, index);
			RemoveChar(PartialValue, sep);
			PartialValue = GetVariable(PartialValue, Variables);
			Param = Param.substr(index+1);
			Value += PartialValue;
			RemoveFirstAndEndBlanks(Param);
			if(!Param.length())
				break;
			if(Param[0] != '+')
				break;
			Param = Param.substr(1);
			index = Param.find(sep);
			if(index == string::npos)
				break;
			Param = Param.substr(index+1);
		}
		if(stopit)
			break;

		CleanString(Parameter);
		if(Parameter.length())
			m_Parameters.SetValue(Parameter, Value);
	}
}



/*

void NvServerCommand::ParseParameters(const char *Parameters, NvObjIndStringList* Variables)
{
	m_Parameters.RemoveAll();

	if(Parameters == NULL)
		return;
	
	string Param = Parameters;
	
	CleanString(Param);

	while(1)
	{
		string Parameter = "";
		string Value = "";
		string::size_type index = Param.find('=');
		if(index == string::npos)
			break;
		Parameter = Param.substr(0, index);
		Param = Param.substr(index+1);

		RemoveFirstAndEndBlanks(Parameter);

		index = Param.find('"');
		if(index == string::npos)
			break;
		Param = Param.substr(index+1);
		
		index = 0;
		while(1)
		{
			if(index >= Param.size())
			{
				index = string::npos;
				break;
			}
			index = Param.find('"', index);
			if(index == string::npos)
				break;
			if((index+1) >= Param.size())
				break;
			if(Param[index+1] != '"')
				break;
			index++;
			index++;
		}
		if(index == string::npos)
			break;
		Value = Param.substr(0, index);
		RemoveDoubleQuotes(Value);

		Param = Param.substr(index+1);
		if(Parameter.length())
			m_Parameters.SetValue(Parameter, GetVariable(Value, Variables));
	}
}

*/



string NvServerCommand::GetVariable(string Value, NvObjIndStringList* Variables)
{
	if(Variables == NULL)
		return Value;

	string RetVal = Value;
	string ValueHere = Value;
	CleanString(ValueHere);
	MakeUpper(ValueHere);
	if(ValueHere.length() < 2)
		return RetVal;
	if(ValueHere[0] != '#')
		return RetVal;
	if(ValueHere[1] == '#')
	{
		// The string starts with a real # character
		// We must eliminate the double #
		string::size_type index;
		index = RetVal.find('#');
		RetVal.erase(index, 1);
		return RetVal;
	}

	// The Value is a variable content
	ValueHere.erase(0, 1);	// We eliminate the # character
	RetVal = Variables->GetValue(ValueHere);

	return RetVal;
}

// Used for result command encoding
void EncodeCommandResult(int iEncoding, string& StringToEncode, void* pRequest)
{
	if(iEncoding == ENCODING_NONE)
		return;

	if(!StringToEncode.length())
		return;

	NvRequest *RequestHere = (NvRequest *)pRequest;
	bool bNirvaUnicode = RequestHere->b_ClientUnicode;
	if(bNirvaUnicode && (iEncoding == ENCODING_UTF8))
		return;
	if((!bNirvaUnicode) && (iEncoding == ENCODING_LATIN1))
		return;

	bool bCommandUnicode = false;
	if(iEncoding == ENCODING_UTF8)
		bCommandUnicode = true;
	EncodeString(StringToEncode, bNirvaUnicode, bCommandUnicode);
}


void EncodeCommandInputString(int iEncoding, string& StringToEncode, void* pRequest)
{	
	if(!StringToEncode.length())
		return;
	if(iEncoding == ENCODING_NONE)
		return;

	NvRequest *RequestHere = (NvRequest *)pRequest;
	bool bNirvaUnicode = RequestHere->b_ClientUnicode;
	if(bNirvaUnicode && (iEncoding == ENCODING_UTF8))
		return;
	if((!bNirvaUnicode) && (iEncoding == ENCODING_LATIN1))
		return;

	bool bCommandUnicode = false;
	if(iEncoding == ENCODING_UTF8)
		bCommandUnicode = true;
	EncodeString(StringToEncode, bCommandUnicode, bNirvaUnicode);
}

