// ***************************************************************************************
// File: nvmq.h
// Description: interface to websphere mq
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: This file contains the declaration of the exported functions of
//           the nvmq library.
// ***************************************************************************************

#ifndef _EMQ_H
#define _EMQ_H

#ifndef NVSTDCALL
#ifdef WIN32
#define NVSTDCALL __stdcall
#else
#define NVSTDCALL
#endif
#endif

#include <string>

// Message types
#define NVMQ_MSG_UNKNOWN	0
#define NVMQ_MSG_XML		1
#define NVMQ_MSG_SOAP		2
#define NVMQ_MSG_WEBS		3
#define NVMQ_MSG_BINARY		4
#define NVMQ_MSG_FILE		5
#define NVMQ_MSG_COMMAND	6


#define NVMQ_MIN_MESSAGE_LENGTH		12
#define NVMQ_MAX_TYPE_LENGTH		32

// Persistence types
#define NVMQ_PERSISTENCE_NONE		0
#define NVMQ_PERSISTENCE_YES		1
#define NVMQ_PERSISTENCE_DEFAULT	2

// Expiry
#define NVMQ_EXPIRY_NONE		-1

class NvMqInMsg
{
public:
	virtual void Clear() = 0 ;
	virtual char* GetData() = 0 ;
	virtual char* SetData(const char* Buffer, int BufferSize) = 0 ;
	virtual int GetDataSize() = 0 ;
	virtual unsigned char* GetMessageId() = 0 ;
	virtual void SetMessageId(const unsigned char* Buffer, int BufferSize) = 0 ;
	virtual unsigned char* GetCorrelationId() = 0 ;
	virtual void SetCorrelationId(const unsigned char* Buffer, int BufferSize) = 0 ;
	virtual void SetReply(bool Value) = 0 ;
	virtual bool GetReply() = 0 ;
	virtual void SetPersistence(int Value) = 0 ;
	virtual int GetPersistence() = 0 ;
	virtual void SetExpiry(int Value) = 0 ;
	virtual int GetExpiry() = 0 ;
	virtual void SetCopyMessageId(bool Value) = 0 ;
	virtual bool GetCopyMessageId() = 0 ;
	virtual void SetCopyCorrelationId(bool Value) = 0 ;
	virtual bool GetCopyCorrelationId() = 0 ;
	virtual void SetReplyQueueName(const char* Value) = 0 ;
	virtual std::string& GetReplyQueueName() = 0 ;
	virtual void SetType(int Value) = 0 ;
	virtual int GetType() = 0 ;
	virtual void SetParameters(const char* Value) = 0 ;
	virtual std::string& GetParameters() = 0 ;
};



class NvMqOutMsg
{
public:
	virtual void Clear() = 0 ;
	virtual char* GetData() = 0 ;
	virtual char* SetData(const char* Buffer, int BufferSize) = 0 ;
	virtual int GetDataSize() = 0 ;
	virtual unsigned char* GetInputMessageId() = 0 ;
	virtual void SetInputMessageId(const unsigned char* Buffer, int BufferSize) = 0 ;
	virtual unsigned char* GetInputCorrelationId() = 0 ;
	virtual void SetInputCorrelationId(const unsigned char* Buffer, int BufferSize) = 0 ;
	virtual void SetPersistence(int Value) = 0 ;
	virtual int GetPersistence() = 0 ;
	virtual void SetExpiry(int Value) = 0 ;
	virtual int GetExpiry() = 0 ;
	virtual void SetCopyMessageId(bool Value) = 0 ;
	virtual bool GetCopyMessageId() = 0 ;
	virtual void SetCopyCorrelationId(bool Value) = 0 ;
	virtual bool GetCopyCorrelationId() = 0 ;
	virtual void SetWithMessageHeader(bool Value) = 0 ;
	virtual bool GetWithMessageHeader() = 0 ;
	virtual void SetQueueName(const char* Value) = 0 ;
	virtual std::string& GetQueueName() = 0 ;
	virtual void SetType(int Value) = 0 ;
	virtual int GetType() = 0 ;
	virtual void SetErrorCode(int Value) = 0 ;
	virtual int GetErrorCode() = 0 ;
	virtual void SetErrorService(const char* Value) = 0 ;
	virtual std::string& GetErrorService() = 0 ;
	virtual void SetErrorClass(const char* Value) = 0 ;
	virtual std::string& GetErrorClass() = 0 ;
	virtual void SetErrorInfo(const char* Value) = 0 ;
	virtual std::string& GetErrorInfo() = 0 ;
	virtual void SetErrorDesc(const char* Value) = 0 ;
	virtual std::string& GetErrorDesc() = 0 ;
};


#define NVMQIDENT void*

// mq library entry points
typedef NVMQIDENT (*MQ_OPEN)(const char* QmName, const char* QiName, const char* QcName, const char* QsAddress, const char* Filter);
typedef void (*MQ_CLOSE)(NVMQIDENT Queue);
typedef int (*MQ_GET)(NVMQIDENT Queue, int TimeOut, void *MsgIn);
typedef int (*MQ_PUT)(NVMQIDENT Queue, void *MsgOut);
typedef int (*INITMQLIB)(const char* NirvaPath);
typedef void (*EXITMQLIB)();


// Exported functions
extern "C"
{
NVMQIDENT NvMqOpen(const char* QmName, const char* QiName, const char* QcName, const char* QsAddress, const char* Filter);
void NvMqClose(NVMQIDENT Queue);
int NvMqGet(NVMQIDENT Queue, int TimeOut, void *MsgIn);
int NvMqPut(NVMQIDENT Queue, void *MsgOut);
int InitMqLib(const char* NirvaPath);
void ExitMqLib();
}


#endif //_EMQ_H
