// ***************************************************************************************
// File: command.cpp
// Description: Nirva command objects
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;


/////////////////////////////////////////////////////////////////////////////
// NvCommand

NvCommand::NvCommand()
{
	Data = NULL;
	DataSize = 0;
	DataFile = "";
	bDeleteDataFile = false;
	m_iCommandSource = NV_SOURCE_CLIENT;
	// The eventual output parameters are case sensitive
	HttpOutHeaders.m_bCaseSensitive = true;
	Clear();
	bDebugCommand = false;
}

NvCommand::~NvCommand()
{
	Clear();
}



void NvCommand::Clear()
{
	Error.SetError();
	m_sService = "SYSTEM";
	m_sClass = "";
	m_sCommand = "";
	m_sInContainer = NV_CONTAINER_DEFAULT_NAME;
	m_sOutContainer = NV_CONTAINER_DEFAULT_NAME;
	m_sLanguage = "";
	m_sHost = "";
	m_sSourceService = "";
	m_iCommandSource = NV_SOURCE_CLIENT;
	iOutputType = OUTPUT_XML;
	iXmlInputType = XML_IN_NONE;
	iXmlCoding = XML_MODEL_NORMAL;				// Use old style by default for compatibility reason
	bWebs = false;
	m_sInputCharset = "";
	m_sXmlNameSpace = "";
	m_sClientAddress = "";		// TCP/IP client address when command comes from http

	
	bLockSession = true;
	m_pSession = NULL;
	m_pParentSession = NULL;
	m_bHasLockedSession = false;
	m_bInitNamedSessionCommand = false;
	m_bExitNamedSessionCommand = false;

	m_sMachineName = "";
	m_sMachineUser = "";

	bClientUnicode = false;
	bFastMode = false;


	Parameters.RemoveAll();
	HttpInHeaders.RemoveAll();
	HttpOutHeaders.RemoveAll();
	FreeData();
}


void NvCommand::FreeData()
{
	if(Data != NULL)	// we delete eventual associated data
		free(Data);
	DataSize = 0;
	Data = NULL;
	if(bDeleteDataFile)
		if(DataFile.length())
			RemoveFile(DataFile.c_str());
	DataFile = "";
}


void NvCommand::ParseParameters(const char *ParametersToParse, NvObjIndStringList* Variables, char sep)
{
	Parameters.RemoveAll();

	bFastMode=false;

	if(ParametersToParse == NULL)
		return;

	char *Parameter = (char *)ParametersToParse;
	char *Value;
	int pos;
	int Length = (int)(strlen(Parameter));
	if(Length == 0)
		return;
	int NumParameters = 0;
	string sValue;
	string sSingleValue;
	bool bHasDoubles = false;

	// Get first real character
	while(Parameter[0]==' ')
	{
		Parameter++;
		Length--;
	}
	
	for(pos=0; pos<Length; pos++)
	{
		// Go to the = character 
		if(Parameter[pos] != '=')
			continue;

		// Get the value
		Value = Parameter+pos+1;
		
		// set the parameter name
		Parameter[pos] = '\0';
		// Clean spaces
		if(!bFastMode)
		{
			Length = (int)(strlen(Parameter));
			while(Length && (Parameter[Length-1] == ' '))
			{
				Length--;
				Parameter[Length] = '\0';
			}
		}
		// Now the parameter name is OK

		sValue = "";

		while(1)
		{
			bHasDoubles = false;
			// Get the first not blank character
			if(!bFastMode)
			{
				for(pos=0; Value[pos]==' '; pos++);
				Value += pos;
			}
			Length = (int)(strlen(Value));
			if(Length < 2)
				break;

			// The separator is the first letter
			sep=Value[0];
			Value++;
			Length--;
			// We go to the next separator that is not doubled
			for(pos=0; pos<Length; pos++)
			{
				if(Value[pos]==sep)
				{
					if(Value[pos+1]!=sep)
						break;
					Value[pos+1]=1;
					bHasDoubles = true;
					pos++;
				}
			}
			Value[pos] = '\0';
			if(Value[0]=='#')
			{
				// This is a variable
				sSingleValue = Value;
				sSingleValue = GetVariable(sSingleValue, Variables);
				sValue += sSingleValue;
			}
			else
			{
				if(bHasDoubles)
				{
					// This is a variable
					sSingleValue = Value;
					string::size_type index;
					// remove line feeds
					while((index = sSingleValue.find(1)) != string::npos)
						sSingleValue.erase(index, 1);
					sValue += sSingleValue;
				}
				else
					sValue += Value;
			}
			
			// Go to the next not blank character
			while((pos != Length) && (Value[++pos]==' '));
			if(Value[pos] == '+')
			{
				Value += pos+1;
				continue;
			}
			else
			{
				if(!bFastMode)
				{
					if(strcmp(Parameter,"NV_FAST") == 0)
						bFastMode = true;
					else
						Parameters.SetValue(Parameter, sValue.c_str());
				}
				else
					Parameters.SetValue(Parameter, sValue.c_str());
				Parameter = Value+pos;
				Length = (int)(strlen(Parameter));
				pos=-1;
				break;
			}
		}
	}

}

/*
void NvCommand::ParseParameters(const char *ParametersToParse, NvObjIndStringList* Variables, char sep)
{
	Parameters.RemoveAll();

	if(ParametersToParse == NULL)
		return;
	
	string Param = ParametersToParse;
	
//	CleanString(Param);
	string::size_type index;
	string Parameter = "";
	string Value = "";
	string PartialValue = "";
	bool stopit = false;

	bool bAutoSep = false;
	if(sep==0)
		bAutoSep = true;

	while(1)
	{
		Parameter = "";
		Value = "";
		index = Param.find('=');
		if(index == string::npos)
			break;
		Parameter = Param.substr(0, index);
		Param = Param.substr(index+1);

		RemoveFirstAndEndBlanks(Parameter);
		RemoveFirstAndEndBlanks(Param);
		if(bAutoSep && Param.length())
			sep = Param[0];

		index = Param.find(sep);
		if(index == string::npos)
			break;
		Param = Param.substr(index+1);
		
		stopit = false;
		while(1)
		{
			index = 0;
			// We search for the next occurence of a " character that is not doubled
			// This marks the end of the value
			while(1)
			{
				if(index >= Param.size())
				{
					index = string::npos;
					stopit = true;
					break;
				}
				index = Param.find(sep, index);
				if(index == string::npos)
				{
					stopit = true;
					break;
				}
				if((index+1) >= Param.size())
					break;
				if(Param[index+1] != sep)
					break;
				index++;
				index++;
			}
			
			if(stopit)
				break;
			PartialValue = Param.substr(0, index);
			RemoveChar(PartialValue, sep);
			PartialValue = GetVariable(PartialValue, Variables);
			Param = Param.substr(index+1);
			Value += PartialValue;
			RemoveFirstAndEndBlanks(Param);
			if(!Param.length())
				break;
			if(Param[0] != '+')
				break;
			Param = Param.substr(1);
			index = Param.find(sep);
			if(index == string::npos)
				break;
			Param = Param.substr(index+1);
		}
		if(stopit)
			break;

		CleanString(Parameter);
		if(Parameter.length())
			Parameters.SetValue(Parameter, Value);
	}
}
*/


void NvCommand::PostParseParameters(NvObjIndStringList* Variables)
{
	if(Variables == NULL)
		return;
	map<string,string>::iterator pos;
	for(pos = Parameters.m_mList.begin(); pos != Parameters.m_mList.end(); ++pos)
	{
		if(pos->second.length() && (pos->second[0]=='#'))
		{
			string Parameter = pos->first;
			string Value = pos->second;
			Parameters.SetValue(Parameter, GetVariable(Value, Variables));
		}
	}
}



string NvCommand::GetVariable(string Value, NvObjIndStringList* Variables)
{
	if(Variables == NULL)
		return Value;

	string RetVal = Value;
	string ValueHere = Value;
	CleanString(ValueHere);
	MakeUpper(ValueHere);
	if(ValueHere.length() < 2)
		return RetVal;
	if(ValueHere[0] != '#')
		return RetVal;
	if(ValueHere[1] == '#')
	{
		// The string starts with a real # character
		// We must eliminate the double #
		string::size_type index;
		index = RetVal.find('#');
		RetVal.erase(index, 1);
		return RetVal;
	}

	// The Value is a variable content
	ValueHere.erase(0, 1);	// We eliminate the # character
	RetVal = Variables->GetValue(ValueHere);

	return RetVal;
}


// Internal function
void NvCommand::GetStringValue(const char *String, char *Buffer, int BufferSize, int *DataSizeHere, int iEncoding)
{
	m_sLastOutputBuffer = "";
	m_sLastOutputBufferEncoded = "";
	if(String == NULL)
	{
		if(DataSizeHere != NULL)
			*DataSizeHere = 0;
		if(Buffer == NULL)
			return;
		if(BufferSize == 0)
			return;
		Buffer[0] = 0;
		return;
	}

	if((Buffer == NULL) && (DataSizeHere == NULL))
		return;	// Nothing to do in this case

	string RetVal = String;
	// Keep a copy of last output buffer
	m_sLastOutputBuffer = RetVal;
	EncodeCommandResult(iEncoding, RetVal);

	if(DataSizeHere != NULL)
		*DataSizeHere = (int)(RetVal.length());

	// Keep a copy of last output buffer
	m_sLastOutputBufferEncoded = RetVal;
	
	if(Buffer == NULL)
		return;


	// Empty buffer by default;
	Buffer[0] = 0;


	if((int)(RetVal.length() + 1) > BufferSize)
		RetVal.erase(BufferSize-1);

	if(RetVal.length())
		strcpy(Buffer, RetVal.c_str());
}


// Internal function
void NvCommand::GetLastOutputBuffer(char *Buffer, int BufferSize)
{
	if((Buffer == NULL) || (BufferSize == 0))
		return;	// Nothing to do in this case

	// Empty buffer by default;
	Buffer[0] = 0;

	// Here we return the encoded version
	string RetVal = m_sLastOutputBufferEncoded;

	if((int)(RetVal.length() + 1) > BufferSize)
		RetVal.erase(BufferSize-1);

	if(RetVal.length())
		strcpy(Buffer, RetVal.c_str());
}


bool NvCommand::Command(const char * CommandString, char *Buffer, int BufferSize, int *DataSizeHere, int CommandSource, NvError *ErrorToKeep, int iEncoding)
{
	if(DataSizeHere != NULL)
		*DataSizeHere = 0;
	if(CommandString == NULL)
	{
		string sMessageInfo = "No command string in command from service ";
		sMessageInfo += m_sService;
		Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, sMessageInfo);
		return false;
	}

	// Check special command for retreiving the last output buffer
	if(strlen(CommandString) == 39)
	{
		string CommandStringHere = CommandString;
		MakeUpper(CommandStringHere);
		if(CommandStringHere == "NV_CMD=|COMMAND:GET_LAST_OUTPUT_BUFFER|")
		{
			GetLastOutputBuffer(Buffer, BufferSize);
			return true;
		}
	}

	NvSession *Session = (NvSession *)m_pSession;
	NvCommand *NewCommand = NVNEW NvCommand;

	string Line = CommandString;
	RemoveFirstAndEndBlanks(Line);
		
	NewCommand->m_iCommandSource = CommandSource;

	// By default, the procedure containers are the same than the original command containers
	NewCommand->m_sHost = m_sHost;
	NewCommand->m_sMachineName = m_sMachineName;
	NewCommand->m_sMachineUser = m_sMachineUser;
	// By default, the procedure containers are the same than the actual command containers
	NewCommand->m_sInContainer = m_sInContainer;
	NewCommand->m_sOutContainer = m_sOutContainer;

	// The new command inherits of the parent session if there is one
	NewCommand->m_pParentSession = m_pParentSession;

	// The service source is the actual one
	NewCommand->m_sSourceService = m_sService;

	NewCommand->ParseParameters(Line.c_str(), &(Session->m_oVariables));

	// Check if we have to revert the containers
	bool bReverseContainers = false;
	string sReverseContainers = NewCommand->Parameters.GetValue("NV_REVERSE_CONTAINER");
	CleanString(sReverseContainers);
	MakeUpper(sReverseContainers);
	if(sReverseContainers == "YES")
	{
		// Revert the containers
		NewCommand->m_sInContainer = m_sOutContainer;
		NewCommand->m_sOutContainer = m_sInContainer;
	}
	
	// The language is also the same
	NewCommand->m_sLanguage = m_sLanguage;
	NewCommand->HttpInHeaders.CopyFrom(&HttpInHeaders);
	NewCommand->FreeData();
	NewCommand->Error.SetError();

	NewCommand->m_sCommand = NewCommand->Parameters.GetValue("NV_CMD");
	if(NewCommand->m_sCommand.length())
	{
		string::size_type index = NewCommand->m_sCommand.rfind(':');
		if(index != string::npos)
		{
			NewCommand->m_sClass = NewCommand->m_sCommand.substr(0, index);
			NewCommand->m_sCommand = NewCommand->m_sCommand.substr(index+1);
			index = NewCommand->m_sClass.rfind(':');
			if(index != string::npos)
			{
				NewCommand->m_sService = NewCommand->m_sClass.substr(0, index);
				NewCommand->m_sClass = NewCommand->m_sClass.substr(index+1);
			}
		}
	}
	else
	{
		NewCommand->m_sService = NewCommand->Parameters.GetValue("NV_SERVICE");
		NewCommand->m_sClass = NewCommand->Parameters.GetValue("NV_CLASS");
		NewCommand->m_sCommand = NewCommand->Parameters.GetValue("NV_COMMAND");
	}
	CleanString(NewCommand->m_sService, true);
	CleanString(NewCommand->m_sClass, true);
	CleanString(NewCommand->m_sCommand, true);
	if((!NewCommand->m_sService.length()) && (!NewCommand->m_sClass.length()) && (!NewCommand->m_sCommand.length()))
	{
		NewCommand->m_sService = "SYSTEM";
		NewCommand->m_sClass = "MISC";
		NewCommand->m_sCommand = "NOP";
	}
	if(!NewCommand->m_sService.length())
		NewCommand->m_sService = "SYSTEM";
	if(!NewCommand->m_sClass.length())
		NewCommand->m_sClass = NewCommand->m_sService;



	if(NewCommand->Parameters.KeyExist("NV_CONTAINER"))
	{
		NewCommand->m_sInContainer = NewCommand->Parameters.GetValue("NV_CONTAINER");
		NewCommand->m_sOutContainer = NewCommand->Parameters.GetValue("NV_CONTAINER");
	}


	if(NewCommand->Parameters.KeyExist("NV_IN_CONTAINER"))
		NewCommand->m_sInContainer = NewCommand->Parameters.GetValue("NV_IN_CONTAINER");
	if(NewCommand->Parameters.KeyExist("NV_OUT_CONTAINER"))
		NewCommand->m_sOutContainer = NewCommand->Parameters.GetValue("NV_OUT_CONTAINER");

	CleanString(NewCommand->m_sInContainer);
	CleanString(NewCommand->m_sOutContainer);
	CleanString(NewCommand->m_sLanguage);
	MakeUpper(NewCommand->m_sService);				// Service is always uppercase
	MakeUpper(NewCommand->m_sClass);				// Class is always uppercase
	MakeUpper(NewCommand->m_sCommand);				// Command is always uppercase
	MakeLower(NewCommand->m_sInContainer);			// In container is always lowercase
	MakeLower(NewCommand->m_sOutContainer);			// Out container is always lowercase
//	if(!NewCommand->m_sOutContainer.length())
//		NewCommand->m_sOutContainer = NewCommand->m_sInContainer;
	MakeUpper(NewCommand->m_sLanguage);				// language is always uppercase


	if(!NewCommand->m_sInContainer.length())
		NewCommand->m_sInContainer = NV_CONTAINER_DEFAULT_NAME;
	if(!NewCommand->m_sOutContainer.length())
		NewCommand->m_sOutContainer = NV_CONTAINER_DEFAULT_NAME;
		// Remove reserved parameters from command
	NewCommand->Parameters.Remove("NV_CMD");
	NewCommand->Parameters.Remove("NV_SERVICE");
	NewCommand->Parameters.Remove("NV_CLASS");
	NewCommand->Parameters.Remove("NV_COMMAND");
	NewCommand->Parameters.Remove("NV_IN_CONTAINER");
	NewCommand->Parameters.Remove("NV_OUT_CONTAINER");

	if(!NewCommand->m_sService.length())
	{
		string sMessageInfo = "NV_SERVICE in command from service ";
		sMessageInfo += m_sService;
		sMessageInfo += " : ";
		sMessageInfo += Line;
		Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, sMessageInfo);
		delete NewCommand;
		return false;
	}

	if(!NewCommand->m_sCommand.length())
	{
		string sMessageInfo = "NV_COMMAND in command from service ";
		sMessageInfo += m_sService;
		sMessageInfo += " : ";
		sMessageInfo += Line;
		Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_BAD_PARAMETER, sMessageInfo);
		delete NewCommand;
		return false;
	}

	// We execute the command
	bool RetVal = true;
	try
	{
		RetVal = Session->Command(NewCommand, Buffer, BufferSize, DataSizeHere, iEncoding);
	}
	catch(...)
	{
		string Message = NewCommand->m_sService;
		Message += ":";
		Message += NewCommand->m_sClass;
		Message += ":";
		Message += NewCommand->m_sCommand;
		NewCommand->Error.SetError("SYSTEM", "COMMAND", NV_COMMAND_EXCEPTION, Message);
		RetVal = false;
	}
	m_sLastOutputBuffer = NewCommand->m_sLastOutputBuffer;
	m_sLastOutputBufferEncoded = NewCommand->m_sLastOutputBufferEncoded;

	// In case of error, we must copy the error information to actual command object
	
	// Get the no error flag
	string NoError = NewCommand->Parameters.GetValue("NV_NO_ERROR");
	MakeUpper(NoError);
	CleanString(NoError);
	bool bNoError = false;
	if(NoError == "YES")
		bNoError = true;

	if(bNoError)
		if(!RetVal)
			RetVal = true;

	
	// Get the keep error flag
	string KeepError = NewCommand->Parameters.GetValue("NV_KEEP_ERROR");
	MakeUpper(KeepError);
	CleanString(KeepError);
	bool bKeepError = false;
	if(KeepError == "YES")
		bKeepError = true;

	if(bKeepError)
	{
		// We keep the error but only if there is one. Otherwise we set the current error if there is an error in the command
		if(ErrorToKeep != NULL)
			Error.SetError(ErrorToKeep->m_sErrorService, ErrorToKeep->m_sErrorClass, ErrorToKeep->m_iErrorCode, ErrorToKeep->m_sErrorInfo);
		if(Error.m_iErrorCode == 0)
			if(!RetVal)
				Error.SetError(NewCommand->Error.m_sErrorService, NewCommand->Error.m_sErrorClass, NewCommand->Error.m_iErrorCode, NewCommand->Error.m_sErrorInfo);
	}
	else
	{
		if(RetVal)
			Error.SetError();
		else
			Error.SetError(NewCommand->Error.m_sErrorService, NewCommand->Error.m_sErrorClass, NewCommand->Error.m_iErrorCode, NewCommand->Error.m_sErrorInfo);
	}

	delete NewCommand;

	return RetVal;
}

/////////////////////////////////////////////////////////////////////////////
// MyNvServiceCommand

MyNvServiceCommand::MyNvServiceCommand()
{
	m_pCommand = NULL;
	m_sServiceSessionId = "";
	i_Encoding = ENCODING_NONE;

}

MyNvServiceCommand::~MyNvServiceCommand()
{
}


void MyNvServiceCommand::SetError(const char *ErrorClass, int ErrorCode, const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorClassHere = "";
	if(ErrorClass != NULL)
		ErrorClassHere = ErrorClass;
	EncodeCommandInputString(i_Encoding, ErrorClassHere);
	string ErrorInfoHere = "";
	if(ErrorInfo != NULL)
		ErrorInfoHere = ErrorInfo;
	EncodeCommandInputString(i_Encoding, ErrorInfoHere);
	if((ErrorClass != NULL) && (strlen(ErrorClass)))
		RealCommand->Error.SetError(RealCommand->m_sService, ErrorClassHere, ErrorCode, ErrorInfoHere);
	else
		RealCommand->Error.SetError(RealCommand->m_sService, RealCommand->m_sService, ErrorCode, ErrorInfoHere);
}


bool MyNvServiceCommand::Command(const char *CommandString, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string CommandStringHere = "";
	if(CommandString != NULL)
		CommandStringHere = CommandString;
	EncodeCommandInputString(i_Encoding, CommandStringHere);
	bool RetVal = RealCommand->Command(CommandStringHere.c_str(), Buffer, BufferSize, DataSize, NV_SOURCE_SERVICE, NULL, i_Encoding);
	return RetVal;
}


void MyNvServiceCommand::GetSessionId(char *Buffer, int BufferSize, int *DataSize)
{

	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	NvSession *Session = (NvSession *)(RealCommand->m_pSession);
	RealCommand->GetStringValue(Session->m_sIdent.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetClass(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sClass.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetCommand(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sCommand.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetInContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sInContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetOutContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sOutContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetLanguage(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sLanguage.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetMachineName(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sMachineName.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetMachineUser(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sMachineUser.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorType = "CODE";
	if(InfoType != NULL)
		ErrorType = InfoType;
	CleanString(ErrorType);
	MakeUpper(ErrorType);
	string RetVal = "";
	if(ErrorType == "CODE")
	{
		char Buf[50];
		sprintf(Buf, "%d", RealCommand->Error.m_iErrorCode);
		RetVal = Buf;
	}
	if(ErrorType == "SERVICE")
		RetVal = RealCommand->Error.m_sErrorService;
	if(ErrorType == "CLASS")
		RetVal = RealCommand->Error.m_sErrorClass;
	if(ErrorType == "INFO")
		RetVal = RealCommand->Error.m_sErrorInfo;
	if(ErrorType == "DESC")
		RetVal = RealCommand->Error.GetErrorDesc(RealCommand->m_sLanguage.c_str());
	if(ErrorType == "DESCRIPTION")
		RetVal = RealCommand->Error.GetErrorDesc(RealCommand->m_sLanguage.c_str());

	RealCommand->GetStringValue(RetVal.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


int MyNvServiceCommand::GetNumParameters()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->Parameters.GetSize();
}


int MyNvServiceCommand::GetSource()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->m_iCommandSource;
}


bool MyNvServiceCommand::ParameterExist(const char *ParameterName)
{
	if(ParameterName == NULL)
		return false;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);
	if(RealCommand->Parameters.KeyExist(ParameterNameHere))
		return true;
	return false;
}


void MyNvServiceCommand::GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize)
{
	if(ParameterName == NULL)
		return;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);
	RealCommand->GetStringValue(RealCommand->Parameters.GetValue(ParameterNameHere).c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	index--;	// The real index starts at 0
	string Key = RealCommand->Parameters.GetKey(index);
	RealCommand->GetStringValue(Key.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvServiceCommand::AddHttpHeader(const char *HeaderName, const char *HeaderValue)
{
	if(HeaderName == NULL)
		return;
	if(HeaderValue == NULL)
		return;
	string HeaderNameHere = "";
	if(HeaderName != NULL)
		HeaderNameHere = HeaderName;
	EncodeCommandInputString(i_Encoding, HeaderNameHere);
	string HeaderValueHere = "";
	if(HeaderValue != NULL)
		HeaderValueHere = HeaderValue;
	EncodeCommandInputString(i_Encoding, HeaderValueHere);
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->HttpOutHeaders.SetValue(HeaderNameHere.c_str(), HeaderValueHere.c_str());
}


void MyNvServiceCommand::SetServiceSession(void *ServiceSession)
{
	unsigned long iServiceSessionId = (unsigned long)ServiceSession;
	if(ServiceSession == NULL)
	{
		m_sServiceSessionId = "";
		return;
	}

	char sServiceSessionId[20];
	sprintf(sServiceSessionId, "%lu", iServiceSessionId);
	m_sServiceSessionId = sServiceSessionId;
}



void* MyNvServiceCommand::GetServiceSession()
{
	if(!m_sServiceSessionId.length())
		return NULL;
	
	unsigned long iServiceSessionId = strtoul(m_sServiceSessionId.c_str(), NULL, 10);
	return (void *)iServiceSessionId;
}



bool MyNvServiceCommand::IsCommand(const char *Class, const char *Command, int Source)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	if(Class != NULL)
	{
		string ClassHere = Class;
		EncodeCommandInputString(i_Encoding, ClassHere);
		CleanString(ClassHere);
		MakeUpper(ClassHere);
		if(ClassHere != RealCommand->m_sClass)
			return false;
	}
	if(Command != NULL)
	{
		string CommandHere = Command;
		EncodeCommandInputString(i_Encoding, CommandHere);
		CleanString(CommandHere);
		MakeUpper(CommandHere);
		if(CommandHere != RealCommand->m_sCommand)
			return false;
	}
	if(Source != -1)
	{
		if(Source != RealCommand->m_iCommandSource)
			return false;
	}

	return true;
}



/////////////////////////////////////////////////////////////////////////////
// MyNvPerlCommand

MyNvPerlCommand::MyNvPerlCommand()
{
	m_pCommand = NULL;
	i_Encoding = ENCODING_NONE;
}

MyNvPerlCommand::~MyNvPerlCommand()
{
}


void MyNvPerlCommand::SysLog(const char *Message, const char *Extra, int Type, int Level, const char *Source)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string MessageHere = "";
	if(Message != NULL)
		MessageHere = Message;
	EncodeCommandInputString(i_Encoding, MessageHere);
	string ExtraHere = "";
	if(Extra != NULL)
		ExtraHere = Extra;
	EncodeCommandInputString(i_Encoding, ExtraHere);
	string SourceHere = "";
	if(Source != NULL)
		SourceHere = Source;
	EncodeCommandInputString(i_Encoding, SourceHere);
	if(RealCommand != NULL)
	{
		NvSession *Session = (NvSession *)(RealCommand->m_pSession);
		GetMain()->Logs.Write("PERL", MessageHere.c_str(), ExtraHere.c_str(), Type, Level, Session->m_sIdent.c_str(), SourceHere.c_str());
	}
	else
		GetMain()->Logs.Write("PERL", MessageHere.c_str(), ExtraHere.c_str(), Type, Level, "", SourceHere.c_str());
}


int MyNvPerlCommand::GetLogLevel()
{
	return GetMain()->Logs.GetLevel("PERL");
}


void MyNvPerlCommand::SetError(const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorInfoHere = "";
	if(ErrorInfo != NULL)
		ErrorInfoHere = ErrorInfo;
	EncodeCommandInputString(i_Encoding, ErrorInfoHere);

	RealCommand->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_PERL_ERROR, ErrorInfoHere);
	CopyError.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_PERL_ERROR, ErrorInfoHere);
}


void MyNvPerlCommand::SetErrorEx(const char *ErrorService, const char *ErrorClass, int ErrorCode, const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	if(ErrorCode == -1)
		RealCommand->Error.SetError(CopyError.m_sErrorService, CopyError.m_sErrorClass, CopyError.m_iErrorCode, CopyError.m_sErrorInfo);
	else
	{
		string ErrorServiceHere = "";
		if(ErrorService != NULL)
			ErrorServiceHere = ErrorService;
		EncodeCommandInputString(i_Encoding, ErrorServiceHere);
		string ErrorClassHere = "";
		if(ErrorClass != NULL)
			ErrorClassHere = ErrorClass;
		EncodeCommandInputString(i_Encoding, ErrorClassHere);
		string ErrorInfoHere = "";
		if(ErrorInfo != NULL)
			ErrorInfoHere = ErrorInfo;
		EncodeCommandInputString(i_Encoding, ErrorInfoHere);
		RealCommand->Error.SetError(ErrorServiceHere, ErrorClassHere, ErrorCode, ErrorInfoHere);
		CopyError.SetError(ErrorServiceHere, ErrorClassHere, ErrorCode, ErrorInfoHere);
	}
}


bool MyNvPerlCommand::Command(const char *CommandString, char *Buffer, int BufferSize, int *DataSize, bool ControlErrorToScript)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string CommandStringHere = "";
	if(CommandString != NULL)
		CommandStringHere = CommandString;
	EncodeCommandInputString(i_Encoding, CommandStringHere);
	bool RetVal = RealCommand->Command(CommandStringHere.c_str(), Buffer, BufferSize, DataSize, NV_SOURCE_PROCEDURE, &CopyError, i_Encoding);

	CopyError.SetError(RealCommand->Error.m_sErrorService, RealCommand->Error.m_sErrorClass, RealCommand->Error.m_iErrorCode, RealCommand->Error.m_sErrorInfo);
	if(ControlErrorToScript)
		RealCommand->Error.SetError();
	return RetVal;
}


void MyNvPerlCommand::GetSessionId(char *Buffer, int BufferSize, int *DataSize)
{

	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	NvSession *Session = (NvSession *)(RealCommand->m_pSession);
	RealCommand->GetStringValue(Session->m_sIdent.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetService(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sService.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetClass(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sClass.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetCommand(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sCommand.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetInContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sInContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetOutContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sOutContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetLanguage(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sLanguage.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorType = "CODE";
	if(InfoType != NULL)
		ErrorType = InfoType;
	CleanString(ErrorType);
	MakeUpper(ErrorType);
	string RetVal = "";
	if(ErrorType == "CODE")
	{
		char Buf[50];
		sprintf(Buf, "%d", CopyError.m_iErrorCode);
		RetVal = Buf;
	}
	if(ErrorType == "SERVICE")
		RetVal = CopyError.m_sErrorService;
	if(ErrorType == "CLASS")
		RetVal = CopyError.m_sErrorClass;
	if(ErrorType == "INFO")
		RetVal = CopyError.m_sErrorInfo;
	if(ErrorType == "DESC")
		RetVal = CopyError.GetErrorDesc(RealCommand->m_sLanguage.c_str());
	if(ErrorType == "DESCRIPTION")
		RetVal = CopyError.GetErrorDesc(RealCommand->m_sLanguage.c_str());

	RealCommand->GetStringValue(RetVal.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


int MyNvPerlCommand::GetNumParameters()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->Parameters.GetSize();
}


int MyNvPerlCommand::GetSource()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->m_iCommandSource;
}


bool MyNvPerlCommand::ParameterExist(const char *ParameterName)
{
	if(ParameterName == NULL)
		return false;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);

	if(RealCommand->Parameters.KeyExist(ParameterNameHere))
		return true;
	return false;
}


void MyNvPerlCommand::GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize)
{
	if(ParameterName == NULL)
		return;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);
	RealCommand->GetStringValue(RealCommand->Parameters.GetValue(ParameterName).c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvPerlCommand::GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	index--;	// The real index starts at 0
	string Key = RealCommand->Parameters.GetKey(index);
	RealCommand->GetStringValue(Key.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}



void MyNvPerlCommand::SetEncoding(const char *Encoding)
{
	if(Encoding == NULL)
		return;
	string sEncoding = Encoding;
	CleanString(sEncoding);
	MakeUpper(sEncoding);
	if(sEncoding == "NONE")
		i_Encoding = ENCODING_NONE;
	if(sEncoding == "UTF-8")
		i_Encoding = ENCODING_UTF8;
	if(sEncoding == "ISO-8859-1")
		i_Encoding = ENCODING_LATIN1;
}






/////////////////////////////////////////////////////////////////////////////
// MyNvJavaCommand

MyNvJavaCommand::MyNvJavaCommand()
{
	m_pCommand = NULL;
	i_Encoding = ENCODING_UTF8;		// Always UTF8 for java
}

MyNvJavaCommand::~MyNvJavaCommand()
{
}


void MyNvJavaCommand::SetError(const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorInfoHere = "";
	if(ErrorInfo != NULL)
		ErrorInfoHere = ErrorInfo;
	EncodeCommandInputString(i_Encoding, ErrorInfoHere);

	RealCommand->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_JAVA_ERROR, ErrorInfoHere);
	CopyError.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_JAVA_ERROR, ErrorInfoHere);
}



void MyNvJavaCommand::SysLog(const char *Message, const char *Extra, int Type, int Level, const char *Source)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string MessageHere = "";
	if(Message != NULL)
		MessageHere = Message;
	EncodeCommandInputString(i_Encoding, MessageHere);
	string ExtraHere = "";
	if(Extra != NULL)
		ExtraHere = Extra;
	EncodeCommandInputString(i_Encoding, ExtraHere);
	string SourceHere = "";
	if(Source != NULL)
		SourceHere = Source;
	EncodeCommandInputString(i_Encoding, SourceHere);
	if(RealCommand != NULL)
	{
		NvSession *Session = (NvSession *)(RealCommand->m_pSession);
		GetMain()->Logs.Write("JAVA", MessageHere.c_str(), ExtraHere.c_str(), Type, Level, Session->m_sIdent.c_str(), SourceHere.c_str());
	}
	else
		GetMain()->Logs.Write("JAVA", MessageHere.c_str(), ExtraHere.c_str(), Type, Level, "", SourceHere.c_str());
}


int MyNvJavaCommand::GetLogLevel()
{
	return GetMain()->Logs.GetLevel("JAVA");
}


void MyNvJavaCommand::SetErrorEx(const char *ErrorService, const char *ErrorClass, int ErrorCode, const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	if(ErrorCode == -1)
		RealCommand->Error.SetError(CopyError.m_sErrorService, CopyError.m_sErrorClass, CopyError.m_iErrorCode, CopyError.m_sErrorInfo);
	else
	{
		string ErrorServiceHere = "";
		if(ErrorService != NULL)
			ErrorServiceHere = ErrorService;
		EncodeCommandInputString(i_Encoding, ErrorServiceHere);
		string ErrorClassHere = "";
		if(ErrorClass != NULL)
			ErrorClassHere = ErrorClass;
		EncodeCommandInputString(i_Encoding, ErrorClassHere);
		string ErrorInfoHere = "";
		if(ErrorInfo != NULL)
			ErrorInfoHere = ErrorInfo;
		EncodeCommandInputString(i_Encoding, ErrorInfoHere);
		RealCommand->Error.SetError(ErrorServiceHere, ErrorClassHere, ErrorCode, ErrorInfoHere);
		CopyError.SetError(ErrorServiceHere, ErrorClassHere, ErrorCode, ErrorInfoHere);
	}
}

bool MyNvJavaCommand::Command(const char *CommandString, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string CommandStringHere = "";
	if(CommandString != NULL)
		CommandStringHere = CommandString;
	EncodeCommandInputString(i_Encoding, CommandStringHere);
	bool RetVal = RealCommand->Command(CommandStringHere.c_str(), Buffer, BufferSize, DataSize, NV_SOURCE_PROCEDURE, &CopyError, i_Encoding);

	CopyError.SetError(RealCommand->Error.m_sErrorService, RealCommand->Error.m_sErrorClass, RealCommand->Error.m_iErrorCode, RealCommand->Error.m_sErrorInfo);
	RealCommand->Error.SetError();
	return RetVal;

}


void MyNvJavaCommand::GetSessionId(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	NvSession *Session = (NvSession *)(RealCommand->m_pSession);
	RealCommand->GetStringValue(Session->m_sIdent.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetService(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sService.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetClass(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sClass.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetCommand(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sCommand.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetInContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sInContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetOutContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sOutContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetLanguage(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sLanguage.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorType = "CODE";
	if(InfoType != NULL)
		ErrorType = InfoType;
	CleanString(ErrorType);
	MakeUpper(ErrorType);
	string RetVal = "";
	if(ErrorType == "CODE")
	{
		char Buf[50];
		sprintf(Buf, "%d", CopyError.m_iErrorCode);
		RetVal = Buf;
	}
	if(ErrorType == "SERVICE")
		RetVal = CopyError.m_sErrorService;
	if(ErrorType == "CLASS")
		RetVal = CopyError.m_sErrorClass;
	if(ErrorType == "INFO")
		RetVal = CopyError.m_sErrorInfo;
	if(ErrorType == "DESC")
		RetVal = CopyError.GetErrorDesc(RealCommand->m_sLanguage.c_str());
	if(ErrorType == "DESCRIPTION")
		RetVal = CopyError.GetErrorDesc(RealCommand->m_sLanguage.c_str());

	RealCommand->GetStringValue(RetVal.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


int MyNvJavaCommand::GetNumParameters()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->Parameters.GetSize();
}


int MyNvJavaCommand::GetSource()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->m_iCommandSource;
}


bool MyNvJavaCommand::ParameterExist(const char *ParameterName)
{
	if(ParameterName == NULL)
		return false;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);

	if(RealCommand->Parameters.KeyExist(ParameterNameHere))
		return true;
	return false;
}


void MyNvJavaCommand::GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize)
{
	if(ParameterName == NULL)
		return;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);
	RealCommand->GetStringValue(RealCommand->Parameters.GetValue(ParameterName).c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvJavaCommand::GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	index--;	// The real index starts at 0
	string Key = RealCommand->Parameters.GetKey(index);
	RealCommand->GetStringValue(Key.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}





/////////////////////////////////////////////////////////////////////////////
// MyNvDotnetCommand

MyNvDotnetCommand::MyNvDotnetCommand()
{
	m_pCommand = NULL;
	i_Encoding = ENCODING_UTF8;		// Always UTF8 for dotnet
}

MyNvDotnetCommand::~MyNvDotnetCommand()
{
}


void MyNvDotnetCommand::SetError(const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorInfoHere = "";
	if(ErrorInfo != NULL)
		ErrorInfoHere = ErrorInfo;
	EncodeCommandInputString(i_Encoding, ErrorInfoHere);

	RealCommand->Error.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_DOTNET_ERROR, ErrorInfoHere);
	CopyError.SetError("SYSTEM", "PROCEDURE", NV_PROCEDURE_DOTNET_ERROR, ErrorInfoHere);
}



void MyNvDotnetCommand::SysLog(const char *Message, const char *Extra, int Type, int Level, const char *Source)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string MessageHere = "";
	if(Message != NULL)
		MessageHere = Message;
	EncodeCommandInputString(i_Encoding, MessageHere);
	string ExtraHere = "";
	if(Extra != NULL)
		ExtraHere = Extra;
	EncodeCommandInputString(i_Encoding, ExtraHere);
	string SourceHere = "";
	if(Source != NULL)
		SourceHere = Source;
	EncodeCommandInputString(i_Encoding, SourceHere);
	if(RealCommand != NULL)
	{
		NvSession *Session = (NvSession *)(RealCommand->m_pSession);
		GetMain()->Logs.Write("DOTNET", MessageHere.c_str(), ExtraHere.c_str(), Type, Level, Session->m_sIdent.c_str(), SourceHere.c_str());
	}
	else
		GetMain()->Logs.Write("DOTNET", MessageHere.c_str(), ExtraHere.c_str(), Type, Level, "", SourceHere.c_str());
}


int MyNvDotnetCommand::GetLogLevel()
{
	return GetMain()->Logs.GetLevel("DOTNET");
}


void MyNvDotnetCommand::SetErrorEx(const char *ErrorService, const char *ErrorClass, int ErrorCode, const char *ErrorInfo)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	if(ErrorCode == -1)
		RealCommand->Error.SetError(CopyError.m_sErrorService, CopyError.m_sErrorClass, CopyError.m_iErrorCode, CopyError.m_sErrorInfo);
	else
	{
		string ErrorServiceHere = "";
		if(ErrorService != NULL)
			ErrorServiceHere = ErrorService;
		EncodeCommandInputString(i_Encoding, ErrorServiceHere);
		string ErrorClassHere = "";
		if(ErrorClass != NULL)
			ErrorClassHere = ErrorClass;
		EncodeCommandInputString(i_Encoding, ErrorClassHere);
		string ErrorInfoHere = "";
		if(ErrorInfo != NULL)
			ErrorInfoHere = ErrorInfo;
		EncodeCommandInputString(i_Encoding, ErrorInfoHere);
		RealCommand->Error.SetError(ErrorServiceHere, ErrorClassHere, ErrorCode, ErrorInfoHere);
		CopyError.SetError(ErrorServiceHere, ErrorClassHere, ErrorCode, ErrorInfoHere);
	}
}

bool MyNvDotnetCommand::Command(const char *CommandString, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string CommandStringHere = "";
	if(CommandString != NULL)
		CommandStringHere = CommandString;
	EncodeCommandInputString(i_Encoding, CommandStringHere);
	bool RetVal = RealCommand->Command(CommandStringHere.c_str(), Buffer, BufferSize, DataSize, NV_SOURCE_PROCEDURE, &CopyError, i_Encoding);

	CopyError.SetError(RealCommand->Error.m_sErrorService, RealCommand->Error.m_sErrorClass, RealCommand->Error.m_iErrorCode, RealCommand->Error.m_sErrorInfo);
	RealCommand->Error.SetError();
	return RetVal;

}


void MyNvDotnetCommand::GetSessionId(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	NvSession *Session = (NvSession *)(RealCommand->m_pSession);
	RealCommand->GetStringValue(Session->m_sIdent.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetService(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sService.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetClass(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sClass.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetCommand(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sCommand.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetInContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sInContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetOutContainer(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sOutContainer.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetLanguage(char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	RealCommand->GetStringValue(RealCommand->m_sLanguage.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ErrorType = "CODE";
	if(InfoType != NULL)
		ErrorType = InfoType;
	CleanString(ErrorType);
	MakeUpper(ErrorType);
	string RetVal = "";
	if(ErrorType == "CODE")
	{
		char Buf[50];
		sprintf(Buf, "%d", CopyError.m_iErrorCode);
		RetVal = Buf;
	}
	if(ErrorType == "SERVICE")
		RetVal = CopyError.m_sErrorService;
	if(ErrorType == "CLASS")
		RetVal = CopyError.m_sErrorClass;
	if(ErrorType == "INFO")
		RetVal = CopyError.m_sErrorInfo;
	if(ErrorType == "DESC")
		RetVal = CopyError.GetErrorDesc(RealCommand->m_sLanguage.c_str());
	if(ErrorType == "DESCRIPTION")
		RetVal = CopyError.GetErrorDesc(RealCommand->m_sLanguage.c_str());

	RealCommand->GetStringValue(RetVal.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


int MyNvDotnetCommand::GetNumParameters()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->Parameters.GetSize();
}


int MyNvDotnetCommand::GetSource()
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	return RealCommand->m_iCommandSource;
}


bool MyNvDotnetCommand::ParameterExist(const char *ParameterName)
{
	if(ParameterName == NULL)
		return false;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);

	if(RealCommand->Parameters.KeyExist(ParameterNameHere))
		return true;
	return false;
}


void MyNvDotnetCommand::GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize)
{
	if(ParameterName == NULL)
		return;
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	string ParameterNameHere = "";
	if(ParameterName != NULL)
		ParameterNameHere = ParameterName;
	EncodeCommandInputString(i_Encoding, ParameterNameHere);
	RealCommand->GetStringValue(RealCommand->Parameters.GetValue(ParameterName).c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}


void MyNvDotnetCommand::GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize)
{
	NvCommand *RealCommand = (NvCommand *)m_pCommand;
	index--;	// The real index starts at 0
	string Key = RealCommand->Parameters.GetKey(index);
	RealCommand->GetStringValue(Key.c_str(), Buffer, BufferSize, DataSize, i_Encoding);
}



// Used for result command encoding
void EncodeCommandResult(int iEncoding, string& StringToEncode)
{
	if(iEncoding == ENCODING_NONE)
		return;

	if(!StringToEncode.length())
		return;

	bool bNirvaUnicode = GetMain()->IsUnicode();
	if(bNirvaUnicode && (iEncoding == ENCODING_UTF8))
		return;
	if((!bNirvaUnicode) && (iEncoding == ENCODING_LATIN1))
		return;

	bool bCommandUnicode = false;
	if(iEncoding == ENCODING_UTF8)
		bCommandUnicode = true;
	EncodeString(StringToEncode, bNirvaUnicode, bCommandUnicode);
}


void EncodeCommandInputString(int iEncoding, string& StringToEncode)
{	
	if(!StringToEncode.length())
		return;
	if(iEncoding == ENCODING_NONE)
		return;

	bool bNirvaUnicode = GetMain()->IsUnicode();
	if(bNirvaUnicode && (iEncoding == ENCODING_UTF8))
		return;
	if((!bNirvaUnicode) && (iEncoding == ENCODING_LATIN1))
		return;

	bool bCommandUnicode = false;
	if(iEncoding == ENCODING_UTF8)
		bCommandUnicode = true;
	EncodeString(StringToEncode, bCommandUnicode, bNirvaUnicode);
}

