//*****************************************************************************
// File: error.h
// Description: declarations of the Nirva error classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _ERROR_H
#define _ERROR_H



// Classes declared in this file
class NvError;


#include "task.h"
#include <string>


// This file defines the SYSTEM service error codes

#define NV_OK										0		// No error

// SYSTEM ERROR class
#define NV_SYSTEM_CANNOT_STOP_SERVER				101		// Cannot stop server when running server as a window service
#define NV_SYSTEM_CANNOT_GET_INFO					102		// Cannot get system information
#define NV_SYSTEM_MAX_SESSIONS						103		// Too much sessions (license limitation)
#define NV_SYSTEM_OVER_TIME							104		// Your license does not allow Nirva to run at this time
#define NV_SYSTEM_NOT_ENOUGH_MEMORY					105		// Not enough memory

// SESSION ERROR class
#define NV_SESSION_NOTEXIST							101		// The session doesn't exist
#define NV_SESSION_CANNOTCREATE						102		// Cannot create the session
#define NV_SESSION_CANNOTLOGIN						103		// Cannot login the session to the application
#define NV_SESSION_IDENTIFICATION_FAILED			104		// The session tries to be accessed by an unauthorized user
#define NV_SESSION_CLOSED							105		// The session has been closed
#define NV_SESSION_STOPPED							106		// The commands has been stopped because of a previous error (stop on error mode)
#define NV_SESSION_NOT_INITIALIZED					107		// The session has not been initialized
#define NV_SESSION_CANNOT_CLOSE						108		// The session cannot be closed
#define NV_SESSION_CANNOT_FIND						109		// Cannot find session (probably expired)
#define NV_SESSION_TOO_MANY							110		// Too many sessions (license limitation)
#define NV_SESSION_OPENPROC_ERROR					111		// Error in the session init procedure
#define NV_SESSION_CANNOT_ATTACH_CONSOLE			112		// Cannot attach console

// OBJECT ERROR class
#define NV_OBJECT_NO_NAME							101		// No object name specified
#define NV_OBJECT_NO_TYPE							102		// No object type specified
#define NV_OBJECT_UNKNOWN_TYPE						103		// The object type is unknown
#define NV_OBJECT_NO_DATA							104		// No data for the object
#define NV_OBJECT_CANNOT_CREATE						105		// Cannot create the object
#define NV_OBJECT_CANNOT_POPULATE					106		// Cannot populate the object
#define NV_OBJECT_CANNOT_GET						107		// Cannot get the object
#define NV_OBJECT_CANNOT_GET_TYPE					108		// Cannot get this kind of object
#define NV_OBJECT_CANNOT_GET_DATA					109		// Cannot get the object data
#define NV_OBJECT_CANNOT_CREATE_TEMP_FILE			110		// Cannot create temporary file name for file object
#define NV_OBJECT_CANNOT_CREATE_OPEN_FILE			111		// Cannot open file for file object
#define NV_OBJECT_EVER_EXIST						112		// The local object ever exist
#define NV_INVALID_OBJECT_INDEX						113		// Invalid object index
#define NV_INVALID_OBJECT							114		// The object doesn't exist
#define NV_SAME_OBJECTS								115		// The objects are the same
#define NV_OBJECTS_NOT_COMPATIBLE					116		// The objects are not compatible
#define NV_OBJECT_CANNOT_COPY						117		// Cannot copy objects
#define NV_INVALID_STRINGLIST_INDEX					118		// Invalid string list index
#define NV_INVALID_INDSTRINGLIST_KEY				119		// Invalid indexed string list key
#define NV_TABLE_SEARCH_ERROR						120		// Error in search table
#define NV_TABLE_CANNOT_SAVE						121		// Error in save table
#define NV_TABLE_CANNOT_LOAD						122		// Error in load table
#define NV_TABLE_CANNOT_IMPORT						123		// Error in import table
#define NV_TABLE_CANNOT_EXPORT						124		// Error in export table
#define NV_TABLE_INVALID_CELL						125		// Invalid cell
#define NV_TABLE_CANNOT_SET_CELL_LINE				126		// Error in set cell line
#define NV_TABLE_CANNOT_INSERT_CELL_LINE			127		// Error in insert cell line
#define NV_TABLE_REMOVE_ROWS_ERROR					128		// Error in remove rows
#define NV_TABLE_CLEAR_ROWS_ERROR					129		// Error in clear rows
#define NV_TABLE_BAD_COLUMN_NAME					130		// Bad column name
#define NV_TABLE_MODIFY_COLUMNS_ERROR				131		// Error in modify columns
#define NV_FILE_CANNOT_CREATE						132		// Cannot create file for file object
#define NV_FILE_CANNOT_GET_SIZE						133		// Cannot get file size
#define NV_FILE_CANNOT_CLEAR						134		// Cannot clear file for file object
#define NV_FILE_CANNOT_APPEND						135		// Cannot append file to file object
#define NV_FILE_CANNOT_SET_PERSISTANT				136		// Cannot set this file object persistant (probably in app work dir)
#define NV_FILE_NO_FILE								137		// No file name in file object
#define NV_BINARY_CANNOT_APPEND						138		// Cannot append binary data
#define NV_BINARY_CANNOT_SAVE						139		// Cannot save binary data to file
#define NV_BINARY_CANNOT_LOAD						140		// Cannot load binary data from file
#define NV_OBJECT_BAD_TYPE							141		// Bad object type
#define NV_TABLE_CANNOT_SELECT_FROM					142		// Error in table select from
#define NV_TABLE_CANNOT_SET_PRIMARY					143		// Cannot set the primary key
#define NV_TABLE_CANNOT_SET_ROW_DATA				144		// Cannot set row data
#define NV_TABLE_CANNOT_JOIN						145		// Cannot join tables
#define NV_TABLE_CANNOT_ADD							146		// Cannot add table
#define NV_TABLE_CANNOT_GET_ROW_INDEX				147		// Cannot get row index from primary value
#define NV_TABLE_CANNOT_ADD_RECORDS					148		// Cannot add records to the table
#define NV_FILE_CANNOT_COMPRESS						149		// Cannot compress file
#define NV_FILE_CANNOT_DECOMPRESS					150		// Cannot decompress file
#define NV_OBJECT_CANNOT_MOVE						151		// Cannot move the object


// CONTAINER ERROR class
#define NV_CONTAINER_CANNOT_CREATE					101		// Cannot create the container
#define NV_CONTAINER_CANNOT_GET						102		// Cannot get the container
#define NV_CONTAINER_CANNOT_COPY					103		// Cannot copy the containers
#define NV_INVALID_CONTAINER_INDEX					104		// Bad container index
#define NV_CONTAINER_CANNOT_EXPORT					105		// Cannot export container
#define NV_CONTAINER_CANNOT_IMPORT					106		// Cannot import container


// COMMAND ERROR class
#define NV_COMMAND_NOT_AVAILABLE					101		// Command not available
#define NV_COMMAND_BAD_PARAMETER					102		// Bad command parameter
#define NV_COMMAND_EXCEPTION						103		// Exception occured in command
#define NV_COMMAND_NOT_AVAILABLE_FROM_SOURCE		104		// Command not available from this source
#define NV_COMMAND_BAD_DATE_FORMAT					105		// Bad date parameter format

// PROCEDURE ERROR class
#define NV_PROCEDURE_CANNOT_GET_FILE				101		// Cannot get procedure file
#define NV_PROCEDURE_INVALID_NAME					102		// Invalid procedure name
#define NV_PROCEDURE_CANNOT_OPEN_FILE				103		// Cannot open procedure file
#define NV_PROCEDURE_BAD_PARAMETER					104		// Bad command parameter in procedure file
#define NV_PROCEDURE_PERL_ERROR						105		// Error in the perl script
#define NV_PROCEDURE_JAVA_ERROR						106		// Error in the java method
#define NV_PROCEDURE_DOTNET_ERROR					107		// Error in the dotnet method


// XML ERROR class
#define NV_XML_CANNOT_GET_OBJECT_DATA				101		// Cannot get object XML data
#define NV_XML_INVALID_XSL_NAME						102		// The Xsl name is not correct
#define NV_XML_CANNOT_GET_XSL_FILE					103		// Cannot get the xsl file
#define NV_XML_NO_XSL_PROCESSOR						104		// No Xsl processor defined
#define NV_XML_INVALID_XSL_PROCESSOR_COMMAND		105		// Invalid Xsl processor command
#define NV_XML_XSL_PROCESSING_ERROR					106		// Error running Xsl processor
#define NV_XML_CANNOT_PARSE							107		// Cannot parse XML document
#define NV_XML_BAD_FORMAT							108		// Bad XML input format
#define NV_XML_ERROR_PARSING_COMMAND				109		// Error parsing command parameters from XML input
#define NV_XML_CANNOT_CONNECT_SERVER				110		// Cannot connect requested server
#define NV_XML_NETWORK_ERROR						111		// Error when sending the XML data to server
#define NV_XML_VALIDATION_ERROR						112		// Invalid XML data
#define NV_XML_UNSUPPORTED_ENCODING					113		// Encoding value not supported

// SOAP ERROR class
#define NV_SOAP_ERROR_PARSING_MESSAGE				101		// Error parsing SOAP message (not well formed SOAP message ?)

// LOCK ERROR class
#define NV_LOCK_CANNOT_LOCK							101		// Cannot lock
#define NV_LOCK_CANNOT_UNLOCK						102		// Cannot unlock
#define NV_LOCK_CANNOT_GET_INFO						103		// Cannot get lock list

// SEMAPHORE ERROR class
#define NV_SEMAPHORE_CANNOT_LOCK					101		// Cannot lock
#define NV_SEMAPHORE_CANNOT_UNLOCK					102		// Cannot unlock
#define NV_SEMAPHORE_CANNOT_REMOVE					103		// Cannot remove
#define NV_SEMAPHORE_CANNOT_GET_INFO				104		// Cannot get semaphore list


// REGISTRY ERROR class
#define NV_REGISTRY_CANNOT_GET_KEY					101		// Cannot get registry key
#define NV_REGISTRY_CANNOT_SET_KEY					102		// Cannot set registry key
#define NV_REGISTRY_CANNOT_REMOVE_KEY				103		// Cannot remove registry key
#define NV_REGISTRY_CANNOT_CREATE_KEY				104		// Cannot create registry key
#define NV_REGISTRY_CANNOT_EXPORT_KEY				105		// Cannot export registry key
#define NV_REGISTRY_CANNOT_IMPORT_KEY				106		// Cannot import registry key
#define NV_REGISTRY_CANNOT_OPEN						107		// Cannot open registry


// PRGM ERROR class
#define NV_PRGM_CANNOT_RUN_PRGM						101		// Cannot run external program
#define NV_PRGM_TIME_OUT_PRGM						102		// Time out condition in external program
#define NV_PRGM_ERROR_PRGM							103		// Error code returned by external program
#define NV_PRGM_BAD_PATH_PARAM						104		// Bad parameter in executable path


// SEQUENCE ERROR class
#define NV_SEQUENCE_NOT_EXIST						101		// The sequence doesn't exist
#define NV_SEQUENCE_CANNOT_SAVE						102		// Cannot save the sequence value to the registry

// SERVICE ERROR class
#define NV_SERVICE_CANNOT_INIT						101		// Cannot initialize service (disabled or invalid service)
#define NV_SERVICE_CANNOT_GET						102		// Cannot get service from service list (disabled or invalid service)
#define NV_SERVICE_CANNOT_INIT_SESSION				103		// The service cannot initialize session
#define NV_SERVICE_SYSTEM_CLASS						104		// The service SYSTEM class is reserved
#define NV_SERVICE_CANNOT_DISABLE					105		// Cannot disable the service (probably still in use)
#define NV_SERVICE_CANNOT_GET_SESSION_LIST			106		// Cannot get the list of service connected sessions
#define NV_SERVICE_CANNOT_MOUNT						107		// Cannot mount the service
#define NV_SERVICE_CANNOT_UNMOUNT					108		// Cannot unmount the service
#define NV_SERVICE_CANNOT_CREATE_SKELETON			109		// Cannot create service skeleton
#define NV_SERVICE_CANNOT_GET_INFO					110		// Cannot get service information
#define NV_SERVICE_CANNOT_CREATE_PACKAGE			111		// Cannot create installation package
#define NV_SERVICE_CANNOT_INSTALL					112		// Cannot install service
#define NV_SERVICE_STOPPED							113		// The service is stopped
#define NV_SERVICE_BAD_PLATFORM						114		// The service package file is not for the actual platform
#define NV_SERVICE_BAD_NAME							115		// Invalid service name
#define NV_SERVICE_UNKNOWN_ERROR					116		// Unknown error in service command


// LICENSE ERROR class
#define NV_LICENSE_CANNOT_LOAD_FILE					101		// Cannot load license file
#define NV_LICENSE_CANNOT_SET_KEY					102		// Cannot set license key
#define NV_LICENSE_CANNOT_REMOVE_KEY				103		// Cannot remove key
#define NV_LICENSE_CANNOT_IMPORT					104		// Cannot import license file
#define NV_LICENSE_CANNOT_EXPORT					105		// Cannot export license file
#define NV_LICENSE_BAD_KEY							106		// Invalid or expired key
#define NV_LICENSE_CANNOT_GET_INFO					107		// Cannot get license information


// APPLICATION ERROR class
#define NV_APPLICATION_CANNOT_CREATE_DIR			101		// Cannot create directory
#define NV_APPLICATION_CANNOT_REMOVE_DIR			102		// Cannot remove directory
#define NV_APPLICATION_CANNOT_GET_DIR				103		// Cannot get directory
#define NV_APPLICATION_CANNOT_REMOVE				104		// Cannot remove application
#define NV_APPLICATION_CANNOT_START					105		// Cannot start application
#define NV_APPLICATION_CANNOT_STOP					106		// Cannot stop application
#define NV_APPLICATION_CANNOT_GET_SESSION_LIST		107		// Cannot get the list of application connected sessions
#define NV_APPLICATION_CANNOT_CREATE				108		// Cannot create application
#define NV_APPLICATION_CANNOT_CREATE_PACKAGE		109		// Cannot create installation package
#define NV_APPLICATION_CANNOT_INSTALL				110		// Cannot install application
#define NV_APPLICATION_CANNOT_GET_INFO				111		// Cannot get application information
#define NV_APPLICATION_BAD_PLATFORM					112		// The application package file is not for the actual platform
#define NV_APPLICATION_NOT_FOUND					113		// The application doesn't exist
#define NV_APPLICATION_BAD_NAME						114		// Invalid application name

// WEBSERVICE ERROR class
#define NV_WEBSERVICE_CANNOT_REMOVE					101		// Cannot remove web service
#define NV_WEBSERVICE_CANNOT_START					102		// Cannot start web service
#define NV_WEBSERVICE_CANNOT_STOP					103		// Cannot stop web service
#define NV_WEBSERVICE_CANNOT_GET_INFO				114		// Cannot get web service information
#define NV_WEBSERVICE_CANNOT_CREATE					105		// Cannot create web service
#define NV_WEBSERVICE_CANNOT_CREATE_PACKAGE			106		// Cannot create installation package
#define NV_WEBSERVICE_CANNOT_INSTALL				107		// Cannot install web service
#define NV_WEBSERVICE_BAD_PLATFORM					108		// The web service package file is not for the actual platform
#define NV_WEBSERVICE_CANNOT_UPDATE					109		// Cannot update the web service content
#define NV_WEBSERVICE_CANNOT_EDIT					110		// Cannot turn the web service to edit mode
#define NV_WEBSERVICE_NOT_IN_EDIT					111		// The web service is not in edit mode
#define NV_WEBSERVICE_OPERATION_CANNOT_ENABLE		112		// Cannot enable web service operation
#define NV_WEBSERVICE_OPERATION_CANNOT_DISABLE		113		// Cannot disable web service operation
#define NV_WEBSERVICE_OPERATION_CANNOT_CREATE		114		// Cannot create web service operation
#define NV_WEBSERVICE_OPERATION_CANNOT_REMOVE		115		// Cannot remove web service operation
#define NV_WEBSERVICE_OPERATION_CANNOT_SETPARAM		116		// Cannot set web service operation parameters
#define NV_WEBSERVICE_OPERATION_CANNOT_SET_SOAP		117		// Cannot set the SOAP input example for this service
#define NV_WEBSERVICE_MESSAGE_CANNOT_CREATE			118		// Cannot create web service message
#define NV_WEBSERVICE_MESSAGE_CANNOT_REMOVE			119		// Cannot remove web service message
#define NV_WEBSERVICE_MESSAGE_CANNOT_SETPARAM		120		// Cannot set web service message parameters
#define NV_WEBSERVICE_MESSAGE_CANNOT_SET_CONTENT	121		// Cannot set web service message content
#define NV_WEBSERVICE_MESSAGE_CANNOT_GET_CONTENT	122		// Cannot get web service message content
#define NV_WEBSERVICE_CANNOT_SET_WSDL				123		// Cannot set the WSDL definition for this service
#define NV_WEBSERVICE_UNKNOWN						124		// Unknown web service
#define NV_WEBSERVICE_NOT_AVAILABLE					125		// The web service is not available
#define NV_WEBSERVICE_CANNOT_GET					126		// Cannot get the web service
#define NV_WEBSERVICE_OPERATION_INVALID				127		// The operation doesn't exist
#define NV_WEBSERVICE_BAD_NAME						128		// Invalid web service name


// TRANSACTION ERROR class
#define NV_TRANSACTION_CANNOT_CREATE				101		// Cannot create transaction
#define NV_TRANSACTION_CANNOT_GET					102		// Cannot get transaction
#define NV_TRANSACTION_CANNOT_SAVE					103		// Cannot save transaction parameters to disk
#define NV_TRANSACTION_CANNOT_CREATE_DIR			104		// Cannot create transaction directory
#define NV_TRANSACTION_CANNOT_SAVE_CONTEXT			105		// Cannot save session context
#define NV_TRANSACTION_CANNOT_RESTART				106		// Cannot restart transaction
#define NV_TRANSACTION_CANNOT_LOAD_CONTEXT			107		// Cannot load session context
#define NV_TRANSACTION_CANNOT_REMOVE				108		// Cannot remove transaction
#define NV_TRANSACTION_CANNOT_START_FROM_TRANS		109		// Cannot start a transaction from another transaction
#define NV_TRANSACTION_CANNOT_GET_INFO				110		// Cannot get transaction information
#define NV_TRANSACTION_CANNOT_VALIDATE				111		// Cannot validate the transaction
#define NV_TRANSACTION_CANNOT_ROLLBACK				112		// Cannot rollback the transaction
#define NV_TRANSACTION_IN_QUEUE						113		// The transaction is in process


// SCHEDULER ERROR class
#define NV_SCHEDULER_EXCEPTION_IN_TASK				101		// Exception occurred when running the task
#define NV_SCHEDULER_INVALID_TASK					102		// Bad task name
#define NV_SCHEDULER_TASK_EVER_EXIST				103		// The task ever exist
#define NV_SCHEDULER_CANNOT_CREATE_TASK				104		// Cannot create task
#define NV_SCHEDULER_CANNOT_REMOVE_TASK				105		// Cannot remove task
#define NV_SCHEDULER_CANNOT_GET_TASK_LIST			106		// Cannot get task list
#define NV_SCHEDULER_CANNOT_GET_TASK_PARAM			107		// Cannot get task parameters


// LISTENER ERROR class
#define NV_LISTENER_EVER_EXIST						101		// A listener with the same name exists
#define NV_LISTENER_CANNOT_CREATE					102		// Cannot create the listener
#define NV_LISTENER_STOP_FIRST						103		// The listener must be stopped first
#define NV_LISTENER_CANNOT_REMOVE					104		// Cannot remove the listener
#define NV_LISTENER_CANNOT_START					105		// Cannot start the listener
#define NV_LISTENER_CANNOT_STOP						106		// Cannot stop the listener
#define NV_LISTENER_CANNOT_SET_PARAM				107		// Cannot set listener parameters
#define NV_LISTENER_CANNOT_GET_LIST					108		// Cannot get listener information


// PACKAGE ERROR class
#define NV_PACKAGE_CANNOT_CREATE_PACKAGE			101		// Cannot create installation package
#define NV_PACKAGE_CANNOT_INSTALL					102		// Cannot install package
#define NV_PACKAGE_BAD_PLATFORM						103		// The package file is not for the actual platform


// LOG ERROR class
#define NV_LOG_INVALID_NAME							101		// Bad log name
#define NV_LOG_CANNOT_CREATE						102		// Cannot create log
#define NV_LOG_CANNOT_GET_INFO						103		// Cannot get log information
#define NV_LOG_SEARCH_ERROR							104		// Error in log search
#define NV_LOG_ERASE_ERROR							105		// Error in log erase


// SECURITY ERROR class
#define NV_SECURITY_BAD_USER						101		// Bad user name
#define NV_SECURITY_CANNOT_GET_CONTEXT				102		// Cannot get the security context for this user
#define NV_SECURITY_BAD_PASSWORD					103		// Invalid password
#define NV_SECURITY_BAD_APPLICATION					104		// The application for checking security is not existing or is disabled
#define NV_SECURITY_NOT_ALLOWED						105		// Security alert. You aren't allowed to perform the operation
#define NV_SECURITY_USER_EXIST						106		// The user ever exist
#define NV_SECURITY_CANNOT_ADD_USER					107		// Cannot add the user
#define NV_SECURITY_CANNOT_LIST_USERS				108		// Cannot get user list
#define NV_SECURITY_INVALID_USER					109		// Invalid user name
#define NV_SECURITY_BAD_PASSWORD_SYNTAX				110		// Bad paswword syntax
#define NV_SECURITY_CANNOT_SET_PASSWORD				111		// Cannot set the password
#define NV_SECURITY_ROLE_EXIST						112		// The role ever exist
#define NV_SECURITY_CANNOT_ADD_ROLE					113		// Cannot add the role
#define NV_SECURITY_CANNOT_LIST_ROLES				114		// Cannot get role list
#define NV_SECURITY_INVALID_ROLE					115		// Invalid role name
#define NV_SECURITY_CANNOT_GET_PERMISSIONS			116		// Cannot get permission list
#define NV_SECURITY_BAD_NEW_PASSWORD				117		// Password confirmation failed
#define NV_SECURITY_CANNOT_REMOVE_DEFAULT			118		// Cannot remove default user
#define NV_SECURITY_CANNOT_DISABLE_USER				119		// Cannot enable or disable user
#define NV_SECURITY_OTHER_SERVER					120		// Security is defined on another application and server
#define NV_SECURITY_CHANGE_PASSWORD					121		// The user must change its password
#define NV_SECURITY_DEDICATED_SERVICE				122		// The security is defined on a dedicated service
#define NV_SECURITY_CANNOT_LOAD_USER_CONTEXT		123		// Cannot load user context
#define NV_SECURITY_CANNOT_SAVE_USER_CONTEXT		124		// Cannot save user context
#define NV_SECURITY_ACCESS_NOT_OWNED_SESSION		125		// Cannot acces a session not owned by the client
#define NV_SECURITY_AUTHENTICATION_ERROR			126		// Authentication error (SSO)

// MAIL ERROR class
#define NV_MAIL_NO_SENDER							101		// No sender defined

// REQUEST ERROR class
#define NV_REQUEST_CANNOT_GET_SERVER_CONNECTION		101		// Cannot get connection to NIRVA server
#define NV_REQUEST_NETWORK_ERROR					102		// Network error
#define NV_REQUEST_BAD_REQUEST_IDENTIFIER			103		// Bad request identifier
#define NV_REQUEST_CANNOT_CREATE					104		// Cannot create new request

// MQ ERROR class
#define NV_MQ_BAD_MESSAGE_TYPE						101		// Bad MQ message type
#define NV_MQ_EVER_EXIST							102		// The queue ever exists
#define NV_MQ_CANNOT_CREATE							103		// Cannot create the queue
#define NV_MQ_INVALID_QUEUE							104		// Unknown queue
#define NV_MQ_QUEUE_RUNNING							105		// This operation requires the queue to be stopped
#define NV_MQ_CANNOT_SET_PARAM						106		// Cannot update queue parameters
#define NV_MQ_CANNOT_REMOVE							107		// Cannot remove queue
#define NV_MQ_CANNOT_START							108		// Cannot start queue
#define NV_MQ_CANNOT_STOP							109		// Cannot stop queue
#define NV_MQ_CANNOT_LIST							110		// Cannot list queue
#define NV_MQ_NOT_RUNNING							111		// The MQ connector is not running

// FILEDB ERROR class
#define NV_FILEDB_CANNOT_OPEN_DB					101		// Cannot open database file
#define NV_FILEDB_NOT_CONNECTED						102		// Database file is not opened
#define NV_FILEDB_TIMEOUT							103		// SQL time out
#define NV_FILEDB_CANNOT_EXECUTE_SQL				104		// SQL error

// HTTP ERROR class
#define NV_HTTP_PROXY_AUTHENTICATION_ERROR			101		// Proxy http authentication error
#define NV_HTTP_AUTHENTICATION_ERROR				102		// Host http authentication error

// THREAD ERROR class
#define NV_THREAD_CANNOT_CREATE						101		// Cannot create the session thread

// RENDERER ERROR class
#define NV_RENDERER_PROCESSING_ERROR				101		// Renderer error

// NIDGETS ERROR class
#define NV_NIDGET_NOT_ENOUGH_MEMORY					101		// Not enough memory
#define NV_NIDGET_CANNOT_BUILD_VIEW					102		// Cannot get the view
#define NV_NIDGET_CANNOT_LOAD_FRAMEWORK				103		// Cannot load the framework
#define NV_NIDGET_VIEW_NOT_FOUND					104		// Cannot find the view
#define NV_NIDGET_DISABLE							105		// Nidget framework is disabled for this application


/////////////////////////////////////////////////////////////////////////////
// NvError class
class NvError
{
public:
	int m_iErrorCode;
	std::string m_sErrorService;
	std::string m_sErrorClass;
	std::string m_sErrorInfo;

	NvError();
	virtual ~NvError();

	void SetError(std::string ErrorService = "", std::string ErrorClass = "", int ErrorCode = NV_OK, std::string ErrorInfo = "");
	std::string GetErrorDesc(const char *Language = NULL);
};




#endif //_ERROR_H
