// ***************************************************************************************
// File: nvjava.h
// Description: Nirva java command interface header
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _NVJAVA_H
#define _NVJAVA_H



// Classes declared in this file
class NvJavaCommand;

#ifndef WIN32
#include <pthread.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#ifdef WIN32
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#include <ctype.h>
#endif


// Command source
#define NV_SOURCE_CLIENT				0		// nvc source
#define NV_SOURCE_BROWSER				1		// web browser source
#define NV_SOURCE_PROCEDURE				2		// procedure source
#define NV_SOURCE_SERVICE				3		// service source
#define NV_SOURCE_SERVER				4		// server source (used in init and close of services)

// log entry type
#define NV_LOG_ALL				-1
#define NV_LOG_INFO				0
#define NV_LOG_WARNING			1
#define NV_LOG_ERROR			2
#define NV_LOG_UNKNOWNTYPE		3



/////////////////////////////////////////////////////////////////////////////
// NvJavaCommand class
// Interface to java
class NvJavaCommand
{
public:
	virtual void SysLog(const char *Message, const char *Extra = NULL, int Type = NV_LOG_INFO, int Level = 1, const char *Source = NULL) = 0 ;
	virtual int GetLogLevel() = 0 ;
	virtual void SetError(const char *ErrorInfo = "") = 0 ;
	virtual void SetErrorEx(const char *ErrorService, const char *ErrorClass, int ErrorCode, const char *ErrorInfo = "") = 0 ;
	virtual bool Command(const char *CommandString, char *Buffer = NULL, int BufferSize = 0, int *DataSize = NULL) = 0 ;
	virtual void GetSessionId(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetService(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetClass(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetCommand(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetInContainer(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetOutContainer(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetLanguage(char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetError(const char *InfoType, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual int GetNumParameters() = 0 ;
	virtual int GetSource() = 0 ;
	virtual bool ParameterExist(const char *ParameterName) = 0 ;
	virtual void GetParameter(const char *ParameterName, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
	virtual void GetParameterName(int index, char *Buffer, int BufferSize, int *DataSize = NULL) = 0 ;
};


#endif //_NVJAVA_H
