// ***************************************************************************************
// File: socket.h
// Description: declarations of the Nirva socket classes
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#ifndef _SOCKET_H
#define _SOCKET_H

#include <openssl/rsa.h>       /* SSLeay stuff */
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>

#include <vector>
#include <string>

// Classes declared in this file
class NvSocket;

// Socket packet size for sending
#define SOCKET_PACKET_SIZE	16384


// Manage client and server sockets
class NvSocket
{

// Construction
public:
	NvSocket();
	NvSocket(unsigned int Block);
	NvSocket(int NewSid, unsigned int Block);

// Attributes
public:
	int Sid;		// socket identifier
	unsigned int Blocking;			// blocking mode
	std::vector<int> m_vPending;	// list of accepted sockets 
	SSL_CTX* ctx;
	SSL* ssl;
	SSL_METHOD *meth;
	bool bSslConnection;
	bool bSSLV3Only;
	bool bAskClientCertificate;
	std::string ClientCAFile;
	std::string CertificateFile;
	std::string CACertFile;
	std::string PrivateKeyFile;
	std::string PrivateKeyPassword;
	std::string LogFile;
	FILE *Logstream;

// Operations
public:
	int WaitRead(unsigned short TimeOut, unsigned short TimeOutMicro = 0);
	int WaitWrite();
	int ReadData(char *Data, int iDataLength, bool Bufferized = false);
	int WriteData(const char *Data, int iDataLength);

	int Listen(unsigned short Port, unsigned short TimeOut, std::vector<int> *Pending, std::vector<void*> *PendingSSL = NULL, std::vector<std::string> *PendingPeerAddr = NULL);
	int Connect(const char *Address, unsigned short Port, unsigned int TimeOut=0);
	int SslInit();


// Implementation
public:
	void Close();
	int InitServerSSL();
	virtual ~NvSocket();

	void WriteLog(const char *Data, int iDataLength = -1);
	void CloseLog();
};


#endif //_SOCKET_H

